@@ -15,7 +15,7 @@ Functions related to printing the player to the screen.
 #define METADATA_MAX_SIZE 256
 #endif
 
-const char VERSION[] = "3.0.2";
+const char VERSION[] = "3.0.3";
 const int ABSOLUTE_MIN_WIDTH = 68;
 bool timeEnabled = true;
 bool drewCover = true;
@@ -60,7 +60,6 @@ int libIter = 0;
 int libSongIter = 0;
 int libTopLevelSongIter = 0;
 
-bool previouslyAllowedChooseSongs = false;
 int previousChosenLibRow = 0;
 int libCurrentDirSongCount = 0;
 FileSystemEntry *lastEntry = NULL;
@@ -69,81 +68,43 @@ const char LIBRARY_FILE[] = "kewlibrary";
 
 FileSystemEntry *library = NULL;
 
-int calcMetadataHeight(void)
-{
-        int term_w, term_h;
-        getTermSize(&term_w, &term_h);
-
-        if (metadata.title[0] != '\0')
-        {
-                size_t titleLength = strnlen(metadata.title, METADATA_MAX_LENGTH);
-                int titleHeight = (int)ceil((float)titleLength / term_w);
-                size_t artistLength = strnlen(metadata.artist, METADATA_MAX_LENGTH);
-                int artistHeight = (int)ceil((float)artistLength / term_w);
-                size_t albumLength = strnlen(metadata.album, METADATA_MAX_LENGTH);
-                int albumHeight = (int)ceil((float)albumLength / term_w);
-                int yearHeight = 1;
-
-                return titleHeight + artistHeight + albumHeight + yearHeight;
-        }
-        else
-        {
-                return 4;
-        }
-}
-
 int calcIdealImgSize(int *width, int *height, const int visualizerHeight, const int metatagHeight)
 {
         float aspectRatio = calcAspectRatio();
 
         int term_w, term_h;
         getTermSize(&term_w, &term_h);
-        int timeDisplayHeight = 1;
-        int heightMargin = 4;
-        int minHeight = visualizerHeight + metatagHeight + timeDisplayHeight + heightMargin;
-        int minBorderWidth = 0;
-        *height = term_h - minHeight;
 
-        double temp_width = ceil((double)(*height) * aspectRatio);
-        if (temp_width > INT_MAX)
-        {
-                *width = INT_MAX;
-        }
-        else if (temp_width < INT_MIN)
-        {
-                *width = INT_MIN;
-        }
-        else
-        {
-                *width = (int)temp_width;
-        }
+        const int timeDisplayHeight = 1;
+        const int heightMargin = 4;
+        const int minHeight = visualizerHeight + metatagHeight + timeDisplayHeight + heightMargin + 1;
+        const int minBorderWidth = 0;
+
+        *height = term_h - minHeight;
+        *width = (int)ceil((double)(*height) * aspectRatio);
 
         if (*width > term_w)
         {
                 *width = term_w - minBorderWidth;
-
-                double temp_height = floor((double)(*width) / aspectRatio);
-                if (temp_height > INT_MAX)
-                {
-                        *height = INT_MAX;
-                }
-                else if (temp_height < INT_MIN)
-                {
-                        *height = INT_MIN;
-                }
-                else
-                {
-                        *height = (int)temp_height;
-                }
+                *height = (int)floor((double)(*width) / aspectRatio);
         }
 
+        if (*width > INT_MAX)
+                *width = INT_MAX;
+        if (*height > INT_MAX)
+                *height = INT_MAX;
+
+
+        *height -= 1;
+
         return 0;
 }
 
 void calcPreferredSize(UISettings *ui)
 {
         minHeight = 2 + (ui->visualizerEnabled ? ui->visualizerHeight : 0);
-        calcIdealImgSize(&preferredWidth, &preferredHeight, (ui->visualizerEnabled ? ui->visualizerHeight : 0), calcMetadataHeight());
+        int metadataHeight = 4;
+        calcIdealImgSize(&preferredWidth, &preferredHeight, (ui->visualizerEnabled ? ui->visualizerHeight : 0), metadataHeight);
 }
 
 void printHelp()
@@ -268,12 +229,11 @@ int displayCover(unsigned char *cover, int coverWidth, int coverHeight, const ch
 {
         if (!ascii)
         {
-                printSquareBitmapCentered(cover, coverWidth, coverHeight, height - 1);
+                printSquareBitmapCentered(cover, coverWidth, coverHeight, height);
         }
         else
         {
-                int width = height * 2;
-                printInAscii(coverArtPath, height - 1, width);
+                printInAscii(coverArtPath, height);
         }
         printf("\n\n");
 
@@ -294,7 +254,7 @@ void printCover(SongData *songdata, UISettings *ui)
         else
         {
                 clearRestOfScreen();
-                for (int i = 0; i < preferredHeight - 1; i++)
+                for (int i = 0; i <= preferredHeight; i++)
                 {
                         printf("\n");
                 }
@@ -333,7 +293,6 @@ void printBasicMetadata(TagSettings const *metadata, UISettings *ui)
         getTermSize(&term_w, &term_h);
         maxWidth = textWidth; // term_w - 3 - (indent * 2);
         printf("\n");
-        setColor(ui);
         int rows = 1;
         if (strnlen(metadata->artist, METADATA_MAX_LENGTH) > 0)
         {
@@ -341,11 +300,16 @@ void printBasicMetadata(TagSettings const *metadata, UISettings *ui)
                 printf(" %.*s\n", maxWidth, metadata->artist);
                 rows++;
         }
+        else {
+                printf("\n");
+        }
         if (strnlen(metadata->album, METADATA_MAX_LENGTH) > 0)
         {
                 printBlankSpaces(indent);
                 printf(" %.*s\n", maxWidth, metadata->album);
-                rows++;
+        }
+        else {
+                printf("\n");
         }
         if (strnlen(metadata->date, METADATA_MAX_LENGTH) > 0)
         {
@@ -355,14 +319,20 @@ void printBasicMetadata(TagSettings const *metadata, UISettings *ui)
                         printf(" %s\n", metadata->date);
                 else
                         printf(" %d\n", year);
-                rows++;
         }
-        cursorJump(rows);
+        else {
+                printf("\n");
+        }
+        cursorJump(4);
         if (strnlen(metadata->title, METADATA_MAX_LENGTH) > 0)
         {
                 PixelData pixel = increaseLuminosity(ui->color, 20);
 
-                if (pixel.r == 255 && pixel.g == 255 && pixel.b == 255)
+                if (ui->useConfigColors)
+                {
+                        setDefaultTextColor();
+                }
+                else if (pixel.r == 255 && pixel.g == 255 && pixel.b == 255)
                 {
                         PixelData gray;
                         gray.r = defaultColor;
@@ -375,12 +345,12 @@ void printBasicMetadata(TagSettings const *metadata, UISettings *ui)
                         printf("\033[1;38;2;%03u;%03u;%03um", pixel.r, pixel.g, pixel.b);
                 }
 
-                if (ui->useConfigColors)
-                        printf("\e[1m\e[39m");
-
                 printWithDelay(metadata->title, 9, maxWidth - 2);
         }
-        cursorJumpDown(rows - 1);
+        else {
+                printf("\n");
+        }
+        cursorJumpDown(3);
 }
 
 int calcElapsedBars(double elapsedSeconds, double duration, int numProgressBars)
@@ -433,7 +403,11 @@ void printMetadata(TagSettings const *metadata, UISettings *ui)
         if (appState.currentView == LIBRARY_VIEW || appState.currentView == PLAYLIST_VIEW || appState.currentView == SEARCH_VIEW)
                 return;
         c_sleep(100);
-        setColor(ui);
+
+        if (ui->useConfigColors)
+                setDefaultTextColor();
+        else
+                setTextColorRGB(ui->color.r, ui->color.g, ui->color.b);
         printBasicMetadata(metadata, ui);
 }
 
@@ -441,8 +415,10 @@ void printTime(double elapsedSeconds, AppState *state)
 {
         if (!timeEnabled || appState.currentView == LIBRARY_VIEW || appState.currentView == PLAYLIST_VIEW || appState.currentView == SEARCH_VIEW)
                 return;
-        if (!state->uiSettings.useConfigColors)
-                setTextColorRGB(state->uiSettings.color.r, state->uiSettings.color.g,state->uiSettings.color.b);
+        if (state->uiSettings.useConfigColors)
+                setDefaultTextColor();
+        else
+                setTextColorRGB(state->uiSettings.color.r, state->uiSettings.color.g, state->uiSettings.color.b);
         int term_w, term_h;
         getTermSize(&term_w, &term_h);
         printBlankSpaces(indent);
@@ -450,13 +426,52 @@ void printTime(double elapsedSeconds, AppState *state)
                 printProgress(elapsedSeconds, duration);
 }
 
+int calcIndentNormal(void)
+{
+        int textWidth = (ABSOLUTE_MIN_WIDTH > preferredWidth) ? ABSOLUTE_MIN_WIDTH : preferredWidth;
+        return getIndentation(textWidth - 1) - 1;
+}
+
+int calcIndentSongView(SongData *songdata)
+{
+        if (songdata == NULL)
+                return calcIndentNormal();
+
+        int titleLength = strnlen(songdata->metadata->title, METADATA_MAX_LENGTH);
+        int albumLength = strnlen(songdata->metadata->album, METADATA_MAX_LENGTH);
+        int maxTextLength = (albumLength > titleLength) ? albumLength : titleLength;
+        textWidth = (ABSOLUTE_MIN_WIDTH > preferredWidth) ? ABSOLUTE_MIN_WIDTH : preferredWidth;
+        int term_w, term_h;
+        getTermSize(&term_w, &term_h);
+        int maxSize = term_w - 2;
+        if (titleLength > 0 && titleLength < maxSize && titleLength > textWidth)
+                textWidth = titleLength;
+        if (maxTextLength > 0 && maxTextLength < maxSize && maxTextLength > textWidth)
+                textWidth = maxTextLength;
+        if (textWidth > maxSize)
+                textWidth = maxSize;
+
+        return getIndentation(textWidth - 1) - 1;
+}
+
+void calcIndent(SongData *songdata)
+{
+        if ((appState.currentView == SONG_VIEW && songdata == NULL) || appState.currentView != SONG_VIEW)
+        {
+                indent = calcIndentNormal();
+        }
+        else {
+                indent = calcIndentSongView(songdata);
+        }
+}
+
 void printGlimmeringText(char *text, int textLength, char *nerdFontText, PixelData color)
 {
         int brightIndex = 0;
         PixelData vbright = increaseLuminosity(color, 120);
         PixelData bright = increaseLuminosity(color, 60);
 
-        printBlankSpaces(indent);
+        printBlankSpaces(calcIndentNormal());
 
         while (brightIndex < textLength)
         {
@@ -502,6 +517,12 @@ void printLastRow(void)
         getTermSize(&term_w, &term_h);
         if (term_w < minWidth)
                 return;
+
+#ifndef __APPLE__
+        // Move to lastRow
+        printf("\033[%d;1H", term_h);
+#endif
+
         setTextColorRGB(lastRowColor.r, lastRowColor.g, lastRowColor.b);
 
 #ifdef __APPLE__
@@ -522,14 +543,26 @@ void printLastRow(void)
 
                 if (isPaused())
                 {
-                        char pauseText[] = " \uf04c";
+                        char pauseText[] = " \u23f8";
+                        snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", pauseText);
+                        currentLength += strnlen(pauseText, maxLength - currentLength);
+                }
+                else if (isStopped())
+                {
+                        char pauseText[] = " \u23f9";
+                        snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", pauseText);
+                        currentLength += strnlen(pauseText, maxLength - currentLength);
+                }
+                else
+                {
+                        char pauseText[] = " \u25b6";
                         snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", pauseText);
                         currentLength += strnlen(pauseText, maxLength - currentLength);
                 }
 
                 if (isRepeatEnabled())
                 {
-                        char repeatText[] = " \uf01e";
+                        char repeatText[] = " \u27f3";
                         snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", repeatText);
                         currentLength += strnlen(repeatText, maxLength - currentLength);
                 }
@@ -577,13 +610,13 @@ void printLastRow(void)
         printf("\033[K"); // clear the line
 
         int textLength = strnlen(text, 100);
-
         int randomNumber = getRandomNumber(1, 808);
+
         if (randomNumber == 808)
                 printGlimmeringText(text, textLength, nerdFontText, lastRowColor);
         else
         {
-                printBlankSpaces(indent);
+                printBlankSpaces(calcIndentNormal());
                 printf("%s", text);
                 printf("%s", nerdFontText);
         }
@@ -606,6 +639,7 @@ int showKeyBindings(SongData *songdata, AppSettings *settings, UISettings *ui)
         int numPrintedRows = 0;
         int term_w, term_h;
         getTermSize(&term_w, &term_h);
+        maxListSize = term_h - 2;
 
         numPrintedRows += printAbout(songdata, ui);
 
@@ -656,9 +690,17 @@ int showKeyBindings(SongData *songdata, AppSettings *settings, UISettings *ui)
         printBlankSpaces(indent);
         printf(" - Esc or %s to quit.\n", settings->quit);
         printf("\n");
-        printLastRow();
 
-        numPrintedRows += 23;
+        numPrintedRows += 22;
+
+        while (numPrintedRows < maxListSize)
+        {
+                printf("\n");
+                numPrintedRows++;
+        }
+
+        printLastRow();
+        numPrintedRows++;
 
         return numPrintedRows;
 }
@@ -869,11 +911,13 @@ void showSearch(SongData *songData, int *chosenRow, UISettings *ui)
 {
         int term_w, term_h;
         getTermSize(&term_w, &term_h);
-        maxSearchListSize = term_h - 5;
+        maxSearchListSize = term_h - 3;
 
         int aboutRows = printLogo(songData, ui);
         maxSearchListSize -= aboutRows;
 
+        setDefaultTextColor();
+
         if (term_w > indent + 38 && !ui->hideHelp)
         {
                 printBlankSpaces(indent);
@@ -902,7 +946,7 @@ void showPlaylist(SongData *songData, PlayList *list, int *chosenSong, int *chos
                 setColor(&state->uiSettings);
 
         printBlankSpaces(indent);
-        printf(" ─ PLAYLIST ─\n");
+        printf("   ─ PLAYLIST ─\n");
         maxListSize -= 1;
 
         displayPlaylist(list, maxListSize, indent, chosenSong, chosenNodeId, state->uiState.resetPlaylistDisplay, state);
@@ -940,64 +984,47 @@ void printVisualizer(double elapsedSeconds, AppState *state)
 {
         UISettings *ui = &(state->uiSettings);
         UIState *uis = &(state->uiState);
+        int term_w, term_h;
+        getTermSize(&term_w, &term_h);
 
         if (ui->visualizerEnabled && appState.currentView == SONG_VIEW)
         {
                 printf("\n");
-                int term_w, term_h;
-                getTermSize(&term_w, &term_h);
+
                 int visualizerWidth = (ABSOLUTE_MIN_WIDTH > preferredWidth) ? ABSOLUTE_MIN_WIDTH : preferredWidth;
                 visualizerWidth = (visualizerWidth < textWidth && textWidth < term_w - 2) ? textWidth : visualizerWidth;
                 visualizerWidth = (visualizerWidth > term_w - 2) ? term_w - 2 : visualizerWidth;
+                visualizerWidth -= 1;
                 uis->numProgressBars = (int)visualizerWidth / 2;
 
+#ifndef __APPLE__
+                saveCursorPosition();
+#endif
                 drawSpectrumVisualizer(ui->visualizerHeight, visualizerWidth, ui->color, indent, ui->useConfigColors);
-
                 printElapsedBars(calcElapsedBars(elapsedSeconds, duration, uis->numProgressBars), uis->numProgressBars);
                 printLastRow();
+#ifndef __APPLE__
+                restoreCursorPosition();
+                cursorJump(1);
+#else
                 int jumpAmount = ui->visualizerHeight + 2;
                 cursorJump(jumpAmount);
-                saveCursorPosition();
+#endif
         }
         else if (!ui->visualizerEnabled)
         {
-                int term_w, term_h;
-                getTermSize(&term_w, &term_h);
                 if (term_w >= minWidth)
                 {
-                        printf("\n\n");
+                        printf("\n");
+
+                        saveCursorPosition();
                         printLastRow();
-                        cursorJump(2);
+                        restoreCursorPosition();
+                        cursorJump(1);
                 }
         }
 }
 
-void calcIndent(SongData *songdata)
-{
-        if (songdata == NULL || appState.currentView != SONG_VIEW)
-        {
-                int textWidth = (ABSOLUTE_MIN_WIDTH > preferredWidth) ? ABSOLUTE_MIN_WIDTH : preferredWidth;
-                indent = getIndentation(textWidth - 1) - 1;
-                return;
-        }
-
-        int titleLength = strnlen(songdata->metadata->title, METADATA_MAX_LENGTH);
-        int albumLength = strnlen(songdata->metadata->album, METADATA_MAX_LENGTH);
-        int maxTextLength = (albumLength > titleLength) ? albumLength : titleLength;
-        textWidth = (ABSOLUTE_MIN_WIDTH > preferredWidth) ? ABSOLUTE_MIN_WIDTH : preferredWidth;
-        int term_w, term_h;
-        getTermSize(&term_w, &term_h);
-        int maxSize = term_w - 2;
-        if (titleLength > 0 && titleLength < maxSize && titleLength > textWidth)
-                textWidth = titleLength;
-        if (maxTextLength > 0 && maxTextLength < maxSize && maxTextLength > textWidth)
-                textWidth = maxTextLength;
-        if (textWidth > maxSize)
-                textWidth = maxSize;
-
-        indent = getIndentation(textWidth - 1) - 1;
-}
-
 FileSystemEntry *getCurrentLibEntry(void)
 {
         return currentEntry;
@@ -1038,6 +1065,7 @@ void processName(const char *name, char *output, int maxWidth)
 
         output[copyLength] = '\0';
         removeUnneededChars(output, copyLength);
+        trim(output, copyLength);
 }
 
 void setChosenDir(FileSystemEntry *entry)
@@ -1091,25 +1119,10 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
                 return false;
         }
 
-        if (chosenLibRow > startLibIter + maxListSize - round(maxListSize / 2))
+        int threshold = startLibIter + (maxListSize + 1) / 2;
+        if (chosenLibRow > threshold)
         {
-                startLibIter = chosenLibRow - maxListSize + round(maxListSize / 2) + 1;
-        }
-
-        if (uis->allowChooseSongs)
-        {
-                if (chosenLibRow >= libIter + libSongIter && libSongIter != 0)
-                {
-                        startLibIter = chosenLibRow - round(maxListSize / 2);
-                }
-        }
-        else
-        {
-                if (chosenLibRow >= 1 + uis->numDirectoryTreeEntries + numTopLevelSongs) // 1 for root
-                {
-                        startLibIter = 1 + uis->numDirectoryTreeEntries + numTopLevelSongs - maxListSize;
-                        chosenLibRow = uis->numDirectoryTreeEntries + numTopLevelSongs;
-                }
+                startLibIter = chosenLibRow - maxListSize / 2 + 1;
         }
 
         if (chosenLibRow < 0)
@@ -1186,7 +1199,7 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
                                                                               (currentEntry != NULL && currentEntry->parent != NULL && chosenDir != NULL && (strcmp(currentEntry->parent->fullPath, chosenDir->fullPath) != 0) &&
                                                                                strcmp(root->fullPath, chosenDir->fullPath) != 0)))
                                         {
-                                                previouslyAllowedChooseSongs = true;
+                                                uis->collapseView = true;
                                                 refresh = true;
 
                                                 if (!uis->openedSubDir)
@@ -1243,7 +1256,7 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
                                 {
                                         filename[0] = '\0';
                                         processName(root->name, filename, maxNameWidth - extraIndent);
-                                        printf("└─%s \n", filename);
+                                        printf("└─ %s\n", filename);
 
                                         libSongIter++;
                                 }
@@ -1303,7 +1316,7 @@ char *getLibraryFilePath(void)
 
 void showLibrary(SongData *songData, AppState *state)
 {
-        if (previouslyAllowedChooseSongs)
+        if (state->uiState.collapseView)
         {
                 if (previousChosenLibRow < chosenLibRow)
                 {
@@ -1317,15 +1330,20 @@ void showLibrary(SongData *songData, AppState *state)
                                 chosenLibRow -= state->uiState.numSongsAboveSubDir;
                                 state->uiState.openedSubDir = false;
                                 state->uiState.numSongsAboveSubDir = 0;
-                                previouslyAllowedChooseSongs = false;
+                                state->uiState.collapseView = false;
                         }
                 }
                 else
                 {
+                        if (state->uiState.openedSubDir)
+                        {
+                                chosenLibRow -= state->uiState.numSongsAboveSubDir;
+                        }
+                        libCurrentDirSongCount = 0;
                         state->uiState.openedSubDir = false;
                         state->uiState.numSongsAboveSubDir = 0;
                 }
-                previouslyAllowedChooseSongs = false;
+                state->uiState.collapseView = false;
         }
 
         UISettings *ui = &(state->uiSettings);
@@ -1376,6 +1394,11 @@ void showLibrary(SongData *songData, AppState *state)
                 refresh = true;
         }
 
+        for (int i = libIter - startLibIter; i < maxLibListSize; i++)
+        {
+                printf("\n");
+        }
+
         printf("\n");
 
         printLastRow();
